#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "screen.h"
#include "main.h"

//Mode0
//BG0 tHg
//SPR J[\

//---------------------------------------------------------------------------
//screen.c
extern ST_SCREEN_BG ScreenBg;

u8 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:   StateInit();   break;
			case STATE_RESET:  StateReset();  break;
			case STATE_INPUT:  StateInput();  break;
			case STATE_YACC:   StateYacc();   break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}


	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();

	SetMode(MODE_0 | BG0_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	//͂̃s[gp
	REG_TM2D     = 65536 - 2621;
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	ScreenBgInit();
	ScreenSprInit();
	ScreenCurInit();

	ScreenBgDrawPanel();
	ScreenCurMoveIn(0, 0);

	State = STATE_INPUT;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInput()
{
	ScreenCurInput();

	State = STATE_YACC;
}
//---------------------------------------------------------------------------
extern void  CalcExec(u8* buf)  __attribute__ ((long_call));
extern char* CalcGetResultBuf() __attribute__ ((long_call));

EWRAM_CODE void StateYacc()
{
	CalcExec(ScreenBg.buf);
	char* p = CalcGetResultBuf();

	ScreenBgDrawAnswer((u8*)p);
	TRACEOUT("Answer: %s\n", p);

	State = STATE_INPUT;
}
