
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "gbfs.h"
#include "lib.h"

#include "screen.h"

//---------------------------------------------------------------------------
ST_BG        Bg0;
ST_SCREEN_BG ScreenBg;
ST_CURSOR    Cursor;

//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgInit()
{
	//tHg
	Bg0.chr = 1;
	Bg0.map = 0;
	Bg0.pal = 0;

	REG_BG0CNT = (BG_SIZEA_256_256     | BG_COLOR_16         |
	              BG_CHARBASE(Bg0.chr) | BG_MAPBASE(Bg0.map) | 0);

	BGSetChrData((u16*)&misakimnTiles, misakimnTilesLen, Bg0.chr);
	BGSetPalData16((u16*)&misakimnPal, Bg0.pal);


	//obt@
	_Memset((u8*)&ScreenBg, 0x00, sizeof(ST_SCREEN_BG));
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgDrawPanel()
{
	BGDrawStr(Bg0.map, 0, 0, SCREEN_BG_CALC_STR);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgDrawBuffer()
{
	BGDrawStr(Bg0.map, 0, 2, ScreenBg.buf);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgDrawAnswer(u8* pStr)
{
	BGDrawStr(Bg0.map, 0, 5, (u8*)"Ans: ");
	BGDrawStr(Bg0.map, 5, 5, pStr);

	u16 i;
	for(i=_Strlen(pStr); i<30; i++)
	{
		BGDrawStr(Bg0.map, 5+i, 5, (u8*)" ");
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgSetBuffer(u8 chr)
{
	ST_SCREEN_BG* p = &ScreenBg;

	if(p->bufCnt+1 >= SCREEN_BG_BUFFER_LEN)
	{
		return;
	}

	p->buf[ p->bufCnt ] = chr;
	p->bufCnt++;
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenBgSetBufferPrev()
{
	ST_SCREEN_BG* p = &ScreenBg;

	if(p->bufCnt == 0)
	{
		return;
	}

	p->bufCnt--;
	p->buf[ p->bufCnt ] = ' ';
}
//---------------------------------------------------------------------------
EWRAM_CODE u8 ScreenBgGetPanelChr(u8 idx)
{
	u8* p = SCREEN_BG_CALC_STR;
	return *(p + idx);
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenSprInit()
{
	SpriteInit();

	SpriteSetPalData16((u16*)&sprPal);
	SpriteSetData((u16*)&sprTiles, sprTilesLen);
}





//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurInit()
{
	ST_CURSOR* p = &Cursor;

	p->x      = 0;
	p->y      = 0;
	p->cx     = 8;
	p->cy     = 8;
	p->sx     = 0;
	p->sy     = 0;
	p->ex     = BG_CX * (_Strlen(SCREEN_BG_CALC_STR) - 1);
	p->ey     = 0;
	p->sprNo  = CURSOR_SPRITE_NO;
	p->tileNo = CURSOR_TILE_NO;

	SpriteSetSize(p->sprNo, SP_SIZE_16, SP_SQUARE, SP_COLOR_16);
	SpriteSetChr(p->sprNo, p->tileNo);

	ScreenCurMoveOut();
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMove()
{
	ST_CURSOR* p = &Cursor;
	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMoveIn(s16 x, s16 y)
{
	ST_CURSOR* p = &Cursor;
	p->x = x;
	p->y = y;

	SpriteMove(p->sprNo, p->x, p->y);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurMoveOut()
{
	ST_CURSOR* p = &Cursor;
	SpriteMove(p->sprNo, 0, SCREEN_CY);
}
//---------------------------------------------------------------------------
EWRAM_CODE void ScreenCurInput()
{
	ST_CURSOR* p = &Cursor;
	u16 idx, chr;
	u16 key;

	for(;;)
	{
		WaitForVsync();
		key = KeyGet();

		if(key == KEY_NONE)
		{
			continue;
		}

		if(key & KEY_R)
		{
			
			return;
		}
		else if(key & KEY_A)
		{
			idx = _UDiv(p->x - p->sx, p->cx);
			chr = ScreenBgGetPanelChr(idx);

			ScreenBgSetBuffer(chr);
			ScreenBgDrawBuffer();
			continue;
		}
		else if(key & KEY_B)
		{
			ScreenBgSetBufferPrev();
			ScreenBgDrawBuffer();
			continue;
		}

		if(key & KEY_UP)
		{
			p->y = MAX(p->y - p->cy, p->sy);
		}
		else if(key & KEY_DOWN)
		{
			p->y = MIN(p->y + p->cy, p->ey);
		}
		else if(key & KEY_LEFT)
		{
			p->x = MAX(p->x - p->cx, p->sx);
		}
		else if(key & KEY_RIGHT)
		{
			p->x = MIN(p->x + p->cx, p->ex);
		}

		ScreenCurMove();
	}
}
